@@ -37,12 +37,27 @@ gt_ = gt  # pylint: disable=invalid-name
 
 
 def subdirs(root: str, start_depth=1, end_depth=1) -> dict:
-    """Get a query string that will look for subdirectories of root optionally specifying the
+    """Get a query string that will look in subdirectories of root optionally specifying the
     start and end depths
     """
-    if end_depth == -1:
-        end_depth = ''  # This will cause the regex to allow any number of repetitions
-    regex = ('^{}([^/]+/){{{},{}}}$'.format(root, start_depth, end_depth))
+    if start_depth in [0, 1] and end_depth == -1:
+        match_root = '^{}'.format(root)
+        and_below = '' if start_depth == 0 else '.+'
+        regex = '{}{}'.format(match_root, and_below)
+    else:
+        if end_depth == -1:
+            end_depth = ''  # This will cause the regex to allow any number of repetitions
+
+        # The breakdown of this regexp is:
+        # ^{root} - match strings beginning with the root
+        # (
+        #   [^/]+ - followed by a '/' or start of string one or more times
+        #   /     - followed by exactly on occurrence of '/'
+        # )
+        # {{{start_depth},{end_depth}}} repeated a minimum of start depth and a maximum of
+        # end_depth times
+        regex = '^{}([^/]+/){{{},{}}}$'.format(root, start_depth, end_depth)
+
     return {pyos.config.DIR_KEY: {'$regex': regex}}
 
 
